@@ -22,7 +22,9 @@
 #include <stdlib.h>
 
 #ifndef VMS
+#ifndef __QNX__
 #include <sys/sysmacros.h>
+#endif
 #include <unistd.h>
 #endif
 
@@ -72,6 +74,8 @@ int deep_opt;
   int ipnt = 0;
   int flagpos, flagval;
 
+  statbuf = statbuf;        /* this shuts up unreferenced compiler warnings */
+
   /* Identify that we are using the SUSP protocol */
   if(deep_opt & NEED_SP){
 	  Rock[ipnt++] ='S';
@@ -160,11 +164,10 @@ int deep_opt;
       cflag = 0;
       if(cpnt1) {
 	*cpnt1 = 0;
-	cflag = SL_CONTINUE;
       };
 
       /* We treat certain components in a special way.  */
-      if(cpnt[0] == '.' && cpnt[1] == '.'){
+      if(cpnt[0] == '.' && cpnt[1] == '.' && cpnt[2] == 0){
 	Rock[ipnt++] = SL_PARENT | cflag;
 	Rock[ipnt++] = 0;  /* length is zero */
 	lenval += 2;
@@ -196,16 +199,26 @@ int deep_opt;
    */
   Rock[ipnt++] ='T';
   Rock[ipnt++] ='F';
+#ifdef __QNX__
+  Rock[ipnt++] = 5 + 4 * 7;
+  Rock[ipnt++] = SU_VERSION;
+  Rock[ipnt++] = 0x0f;
+#else
   Rock[ipnt++] = 5 + 3 * 7;
   Rock[ipnt++] = SU_VERSION;
-  Rock[ipnt++] = 7;
+  Rock[ipnt++] = 0x0e;
+#endif
   flagval |= (1<<7);
-  iso9660_date((char *) &Rock[ipnt], lstatbuf->st_ctime);
+#ifdef __QNX__
+  iso9660_date((char *) &Rock[ipnt], lstatbuf->st_ftime);
   ipnt += 7;
+#endif
   iso9660_date((char *) &Rock[ipnt], lstatbuf->st_mtime);
   ipnt += 7;
   iso9660_date((char *) &Rock[ipnt], lstatbuf->st_atime);
   ipnt += 7;
+  iso9660_date((char *) &Rock[ipnt], lstatbuf->st_ctime);
+  ipnt += 7;
 
   /* 
    * Add in the Rock Ridge RE time field
