@@ -19,10 +19,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-#include "mkisofs.h"
-#include "iso9660.h"
 #include <string.h>
 #include <stdlib.h>
+#include "mkisofs.h"
+#include "iso9660.h"
 #include <time.h>
 #include <errno.h>
 
@@ -94,6 +94,13 @@ void FDECL2(set_733, char *, pnt, unsigned int, i){
 	pnt[4] = pnt[3] = (i >> 24) &  0xff;
 }
 
+static xfwrite(void * buffer, int count, int size, FILE * file)
+{
+  if (!fwrite(buffer, count, size, file) != size) return;
+  fprintf(stderr,"cannot write raw file\n"); 
+  exit(1);
+}
+
 struct deferred_write{
   struct deferred_write * next;
   char * table;
@@ -137,7 +144,11 @@ static void FDECL3(write_one_file, char *, filename, unsigned int, size, FILE *,
   int use;
   int remain;
   if ((infile = fopen(filename, "rb")) == NULL) {
+#ifdef sun
+      fprintf(stderr, "cannot open %s: (%d)\n", filename, errno);
+#else
       fprintf(stderr, "cannot open %s: %s\n", filename, strerror(errno));
+#endif
       exit(1);
   }
   remain = size;
@@ -146,8 +157,11 @@ static void FDECL3(write_one_file, char *, filename, unsigned int, size, FILE *,
 	  use =  (remain >  SECTOR_SIZE * NSECT - 1 ? NSECT*SECTOR_SIZE : remain);
 	  use = ROUND_UP(use); /* Round up to nearest sector boundary */
 	  memset(buffer, 0, use);
-	  fread(buffer, 1, use, infile);
-	  fwrite(buffer, 1, use, outfile);
+	  if (fread(buffer, 1, use, infile) == 0) {
+	    fprintf(stderr,"cannot read from %s\n",filename); 
+	    exit(1);
+	  }
+	  xfwrite(buffer, 1, use, outfile);
 	  last_extent_written += use/SECTOR_SIZE;
 	  if((last_extent_written % 1000) < use/SECTOR_SIZE) fprintf(stderr,"%d..", last_extent_written);
 	  remain -= use;
@@ -160,7 +174,7 @@ static void FDECL1(write_files, FILE *, outfile){
   dwpnt = dw_head;
   while(dwpnt){
 	  if(dwpnt->table) {
-		  fwrite(dwpnt->table,  1, ROUND_UP(dwpnt->size), outfile);
+		  xfwrite(dwpnt->table,  1, ROUND_UP(dwpnt->size), outfile);
 		  table_size += dwpnt->size;
 		  fprintf(stderr,"Size %d ", dwpnt->size);
 		  free(dwpnt->table);
@@ -420,7 +434,7 @@ void FDECL2(generate_one_directory, struct directory *, dpnt, FILE *, outfile){
   if(dpnt->size != dir_index)
     fprintf(stderr,"Unexpected directory length %d %d %s\n",dpnt->size, 
 	    dir_index, dpnt->de_name);
-  fwrite(directory_buffer, 1, total_size, outfile);
+  xfwrite(directory_buffer, 1, total_size, outfile);
   last_extent_written += total_size >> 11;
   free(directory_buffer);
 }
@@ -435,7 +449,7 @@ static void FDECL1(build_pathlist, struct directory *, node){
     if(dpnt->subdir) build_pathlist(dpnt->subdir);
     dpnt = dpnt->next;
   };
-};
+}
 
 int FDECL2(compare_paths, const struct directory **, r, const struct directory **, l) {
   if((*r)->parent->path_index < (*l)->parent->path_index) return -1;
@@ -493,11 +507,6 @@ void generate_path_tables(){
 
 
     namelen = de->isorec.name_len[0];
-    /* I am pretty sure that this is wrong, the namelen should be
-stored with the 721 and 722 macros.  For the intel format path tables
-it does not matter, but the motorola path table name lengths may have
-the bytes reversed.  The Young Minds software does it the way it is coded
-now, but it could easily be wrong. */
 
     path_table_l[path_table_index] = namelen;
     path_table_m[path_table_index] = namelen;
@@ -540,7 +549,7 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
   /* First, we output 16 sectors of all zero */
 
   for(i=0; i<16; i++)
-    fwrite(buffer, 1, sizeof(buffer), outfile);
+    xfwrite(buffer, 1, sizeof(buffer), outfile);
 
   last_extent_written += 16;
 
@@ -551,7 +560,11 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
   vol_desc.version[0] = 1;
 
   memset(vol_desc.system_id, ' ', sizeof(vol_desc.system_id));
+#ifdef __QNX__
+  memcpy(vol_desc.system_id, "QNX", sizeof("QNX"));
+#else
   memcpy(vol_desc.system_id, "LINUX", sizeof("LINUX"));
+#endif
 
   memset(vol_desc.volume_id, ' ', sizeof(vol_desc.volume_id));
   memcpy(vol_desc.volume_id, volid, strlen(volid));
@@ -601,8 +614,8 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
   memcpy(vol_desc.effective_date,  iso_time,  16);
 
   /* For some reason, Young Minds writes this twice.  Aw, what the heck */
-  fwrite(&vol_desc, 1, 2048, outfile);
-  fwrite(&vol_desc, 1, 2048, outfile);
+  xfwrite(&vol_desc, 1, 2048, outfile);
+  xfwrite(&vol_desc, 1, 2048, outfile);
   last_extent_written += 2;
 
   /* Now write the end volume descriptor.  Much simpler than the other one */
@@ -610,15 +623,15 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
   vol_desc.type[0] = ISO_VD_END;
   memcpy(vol_desc.id, ISO_STANDARD_ID, sizeof(ISO_STANDARD_ID));
   vol_desc.version[0] = 1;
-  fwrite(&vol_desc, 1, 2048, outfile);
-  fwrite(&vol_desc, 1, 2048, outfile);
+  xfwrite(&vol_desc, 1, 2048, outfile);
+  xfwrite(&vol_desc, 1, 2048, outfile);
   last_extent_written += 2;
 
   /* Next we write the path tables */
-  fwrite(path_table_l, 1, path_blocks << 11, outfile);
-  fwrite(path_table_l, 1, path_blocks << 11, outfile);
-  fwrite(path_table_m, 1, path_blocks << 11, outfile);
-  fwrite(path_table_m, 1, path_blocks << 11, outfile);
+  xfwrite(path_table_l, 1, path_blocks << 11, outfile);
+  xfwrite(path_table_l, 1, path_blocks << 11, outfile);
+  xfwrite(path_table_m, 1, path_blocks << 11, outfile);
+  xfwrite(path_table_m, 1, path_blocks << 11, outfile);
   last_extent_written += 4*path_blocks;
   free(path_table_l);
   free(path_table_m);
@@ -637,7 +650,7 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
 #endif
   generate_iso9660_directories(root, outfile);
 
-  if(extension_record) fwrite(extension_record, 1, SECTOR_SIZE, outfile);
+  if(extension_record) xfwrite(extension_record, 1, SECTOR_SIZE, outfile);
 
 
   /* Now write all of the files that we need. */
